﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using Props.Shop.Adafruit.Api;
using Props.Shop.Framework;

namespace Props.Shop.Adafruit
{
    public class AdafruitShop : IShop
    {
        private ILoggerFactory loggerFactory;
        private ILogger<AdafruitShop> logger;
        private SearchManager searchManager;
        private Configuration configuration;
        private HttpClient http;
        private bool disposedValue;

        public string ShopName => "Adafruit";

        public string ShopDescription => "A electronic component online hardware company.";

        public string ShopModuleAuthor => "Reslate";

        public SupportedFeatures SupportedFeatures => new SupportedFeatures(
            false,
            false,
            false,
            false,
            true
        );
        public void Initialize(string workspaceDir, ILoggerFactory loggerFactory)
        {
            this.loggerFactory = loggerFactory;
            http = new HttpClient();
            http.BaseAddress = new Uri("http://www.adafruit.com/api/");
            configuration = new Configuration(); 
            // TODO: Implement config persistence.
            // TODO: Implement product listing persisted cache. 
            LiveProductListingManager productListingManager = new LiveProductListingManager(http, loggerFactory.CreateLogger<LiveProductListingManager>(), configuration.MinDownloadInterval);
            this.searchManager = new SearchManager(productListingManager, configuration.Similarity);
            productListingManager.StartUpdateTimer(delay: 0);

            logger = loggerFactory.CreateLogger<AdafruitShop>();
        }

        public Task<ProductListing> GetProductListingFromUrl(string url)
        {
            return searchManager.ProductListingManager.GetProductListingFromUrl(url);
        }

        public IAsyncEnumerable<ProductListing> Search(string query, Filters filters)
        {
            return searchManager.Search(query);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (!disposedValue)
            {
                if (disposing)
                {
                    http.Dispose();
                    searchManager.Dispose();
                }
                disposedValue = true;
            }
        }

        public void Dispose()
        {
            Dispose(disposing: true);
            GC.SuppressFinalize(this);
        }
    }
}
